import unittest

from auth_tkt.helpers import get_ticket_data


class IntegrationTests(unittest.TestCase):
    def setUp(self):
        self.authtkt_secret = 'hmac secret'
        self.crypted_cookie_secret = 'top secret'
        self.cookie = (
            'MjQ4MWQ3ODEzNGI2MjE3N2I4OGQ4MDRjNTZkY2YxZGU1MWRkNjY3NjEyMzQ1Njc4O'
            'TAhdGVzdCFnTGpIMGluemJTZTZHYzNaU2J6S1ljWTZkUlZ5VXloR0pWMWxpWXBYY3'
            'RmTjAxQlRWZFRlOS96M0g0dmEyOUFEWFlDLytFNFpDTmQ5S09OSlBJZnFoWUNnNlF'
            'DYWFDODM3NXdjS1RTbERVUlVUZlV6TUxWZHRZSVpic0JxeGxuRDgyWHBjeE9ORjZB'
            'Y3pvQjlkNkU0N0xScGVVNjNmUTFpdFhOcFkwRExyUG8xdnlGWUtxZHNRTHBYUHc9P'
            'Q==')
        # Encoding of the encrypted data
        self.encoding = 'latin_1'

    def test_get_ticket_with_data(self):
        data = get_ticket_data(self.cookie, self.authtkt_secret,
                               self.crypted_cookie_secret, timeout=0,
                               encoding=self.encoding)
        self.assertEqual(sorted(data), ['id', 'name', 'surname', 'tokens'])

    def test_get_ticket_no_decrypt(self):
        data = get_ticket_data(
            self.cookie, self.authtkt_secret, timeout=0,
            encoding=self.encoding)
        self.assertEqual(sorted(data), ['id', 'tokens'])

    def test_get_ticket_unicode(self):
        data = get_ticket_data(
            u'%s' % self.cookie, u'%s' % self.authtkt_secret,
            u'%s' % self.crypted_cookie_secret, 0, encoding=self.encoding)
        self.assertEqual(sorted(data), ['id', 'name', 'surname', 'tokens'])



import json
import base64
import unittest

from cryptography.hazmat.primitives.ciphers import algorithms

from auth_tkt.ticket import AuthTkt
from auth_tkt.encrypted import (EncryptedAuthTkt, DecryptionError,
                                _derive_keys, _encrypt_userdata,
                                _decrypt_userdata, _encipher,
                                _decipher)


class EncryptedAuthTktTests(unittest.TestCase):
    def test_construction(self):
        tkt = AuthTkt('secret', '123')
        etkt = EncryptedAuthTkt(tkt, 'cryptosecret')
        self.assertEqual(etkt.auth_ticket, tkt)

    def test_from_data_empty(self):
        etkt = EncryptedAuthTkt.from_data('secret', 'cryptosecret', '123')
        self.assertTrue(isinstance(etkt.auth_ticket, AuthTkt))
        self.assertEqual(etkt.data, {})

    def test_from_data_dict(self):
        etkt = EncryptedAuthTkt.from_data('secret', 'cryptosecret', '123',
                                          {'foo': 'bar'})
        self.assertTrue(isinstance(etkt.auth_ticket, AuthTkt))
        self.assertEqual(etkt.data, {'foo': 'bar'})

    def test_tkt_attributes(self):
        tkt = AuthTkt('secret', '123')
        etkt = EncryptedAuthTkt(tkt, 'cryptosecret')
        self.assertEqual(tkt.uid, etkt.uid)
        self.assertEqual(tkt.cookie_value(), etkt.cookie_value())
        self.assertEqual(tkt.ticket(), etkt.ticket())
        self.assertEqual(tkt.cookie('cookie'), etkt.cookie('cookie'))


class DataEncryptionTests(unittest.TestCase):
    cleartext = b'this is a secret message'
    ciphertext = (
        b'tJzLGOp95tK4YMCy+PTmq3vJ/qT+MCFjKJC7GpmLJivlmL1WNeaUTUSp9nV'
        b'FTtyZ419htUd7dZeAM0oHs9Nul6DcR4FxU6U38dYUjMyFtsWsmoMAYQY4PK'
        b'Zivdw+icFIGIyXzUC8HOVfbnh+cIbJEGj+7kvPOvXxKcThxX64usrYbQ==')
    secret = b'cryptosecret'

    def flip_ciphertext_bit(self, byte, bit=0):
        ciphertext = bytearray(base64.b64decode(self.ciphertext))
        ciphertext[byte] = ciphertext[byte] ^ (2 ** bit)
        return base64.b64encode(ciphertext)

    def test_derive_keys_salted(self):
        salt = base64.b64decode(self.ciphertext)[16:48]
        hmackey, enckey, salt = _derive_keys(self.ciphertext, salt)
        expected_hmackey = 'xbNKad7dTFt7wNYCxkFLVwXtMuiF9eCKCXt3oabIPj0='
        expected_enckey = '7IJke6/xvQSdz/L1tFrvkBjHGWVD5hn5WneFaXTytT8='
        self.assertEqual(base64.b64encode(hmackey).decode(), expected_hmackey)
        self.assertEqual(base64.b64encode(enckey).decode(), expected_enckey)

    def test_encrypt_userdata(self):
        ciphertext = _encrypt_userdata(
            self.cleartext, self.secret)
        ciphertext = base64.b64decode(ciphertext)
        # Hand-wavey are-we-encrypted tests
        self.assertTrue(self.cleartext not in ciphertext)
        self.assertTrue(len(ciphertext) >= len(self.cleartext) + 16 + 32)

    def test_decrypt_userdata(self):
        self.assertEqual(
            json.loads(_decrypt_userdata(
                self.ciphertext, self.secret).decode()),
            self.cleartext.decode())

    def test_invalid_contents(self):
        ciphertext = self.flip_ciphertext_bit(byte=128)
        self.assertRaises(DecryptionError, _decrypt_userdata, ciphertext,
                          self.secret)

    def test_derive_keys(self):
        hmackey, enckey, salt = _derive_keys(self.secret)
        self.assertNotEqual(hmackey, enckey)
        self.assertTrue(salt)


class CipherHelperTests(unittest.TestCase):
    plaintext = b'hello there'
    ciphertext = b'Z\xed\xc5\xe8\xb2&\x8b0\x8cK\x05'
    key = b'1' * 16
    iv = b'2' * 16
    alg = algorithms.AES

    def test_encipher(self):
        ciphertext = _encipher(self.plaintext, self.key, self.iv, self.alg)
        self.assertEqual(ciphertext, self.ciphertext)

    def test_decipher(self):
        plaintext = _decipher(self.ciphertext, self.key, self.iv, self.alg)
        self.assertEqual(plaintext, self.plaintext)



from auth_tkt.encrypted import EncryptedAuthTkt
from auth_tkt.ticket import validate


def get_ticket_data(ticket, authtkt_secret, crypted_cookie_secret=None,
                    timeout=7200, encoding='utf-8'):
    """We store user information in our session hashes. You can retreive that
    data with this function."""
    ticket = validate(
        ticket, authtkt_secret, timeout=timeout, encoding=encoding)

    if not ticket:
        return None

    data = {
        'id': ticket.uid,
        'tokens': ticket.tokens,
    }

    if crypted_cookie_secret:
        ticket = EncryptedAuthTkt(ticket, crypted_cookie_secret)
        data.update(ticket.data)

    return data



from base64 import b64decode, b64encode
import sys


IS_PY2 = (sys.version_info.major == 2)


def base64encode(s, encoding='utf-8'):
    if IS_PY2:
        return b64encode(s)
    return b64encode(bytes(s, encoding)).decode()


def base64decode(s, encoding='utf-8'):
    if IS_PY2:
        return b64decode(s)
    return b64decode(s).decode(encoding)


def to_bytes(s, encoding='utf-8'):
    if IS_PY2:
        if isinstance(s, unicode):
            return s.encode(encoding)
        return s
    return bytes(s, encoding)



# Copyright (c) 2009-2013, Yola, Inc.
# Copyright (c) 2005, Imaginary Landscape LLC and Contributors
# The AuthTkt class is derived from Ian Bicking's Python port (now shipped
# with mod_auth_tkt)

import binascii
import hashlib
import socket
import struct
from time import time

try:
    # Python 2
    import Cookie as http_cookies
except ImportError:
    # Python 3
    import http.cookies as http_cookies

from auth_tkt.compat import base64decode, base64encode, to_bytes


DIGEST_HEX_LENGTHS = {
    'md5': len(hashlib.md5(b'').hexdigest()),
    'sha256': len(hashlib.sha256(b'').hexdigest()),
    'sha512': len(hashlib.sha512(b'').hexdigest()),
}


def validate(ticket, secret, ip='0.0.0.0', timeout=7200, encoding='utf-8',
             digest='md5'):
    """Validate a given authtkt ticket for the secret and ip provided"""
    if len(ticket) < 40:
        return False

    raw = ticket
    base64 = False

    if '!' not in ticket:
        try:
            raw = base64decode(ticket, encoding)
            base64 = True
        except binascii.Error:
            return False

    if '!' not in raw:
        return False

    hash_length = DIGEST_HEX_LENGTHS[digest]
    raw = raw[hash_length:]
    ts, raw = raw[:8], raw[8:]
    uid, extra = raw.split('!', 1)
    tokens = data = ''

    try:
        ts = int(ts, 16)
    except ValueError:
        return False

    if timeout and time() - ts > timeout:
        return False

    if extra:
        if '!' in extra:
            tokens, data = extra.split('!', 1)
        else:
            data = extra

    auth_ticket = AuthTkt(
        secret, uid, data, ip, tokens.split(','), base64, ts,
        encoding=encoding, digest=digest)
    if auth_ticket.ticket() == ticket:
        return auth_ticket

    return False


class AuthTkt(object):
    def __init__(self, secret, uid, data='', ip='0.0.0.0', tokens=(),
                 base64=True, ts=None, encoding='utf-8', digest='md5'):
        self.secret = str(secret)
        self.uid = str(uid)
        self.data = data
        self.encoding = encoding
        self.ip = ip
        self.tokens = ','.join(tok.strip() for tok in tokens)
        self.base64 = base64
        self.ts = int(time() if ts is None else ts)
        self.digest = digest

    def ticket(self):
        v = self.cookie_value()
        if self.base64:
            return base64encode(v).strip().replace('\n', '')
        return v

    def cookie(self, name, **kwargs):
        name = str(name)
        c = http_cookies.SimpleCookie()
        c[name] = self.ticket()

        kwargs.setdefault('path', '/')
        c[name].update(kwargs)

        return c

    def cookie_value(self):
        parts = ['%s%08x%s' % (self._digest(), self.ts, self.uid)]
        if self.tokens:
            parts.append(self.tokens)
        parts.append(self.data)
        return '!'.join(parts)

    def _digest(self):
        parts = [self._digest0(), self.secret]
        parts = b''.join([to_bytes(part) for part in parts])
        return hashlib.new(self.digest, parts).hexdigest()

    def _digest0(self):
        parts = (
            self._encode_ip(self.ip), self._encode_ts(self.ts),
            to_bytes(self.secret), to_bytes(self.uid), b'\0',
            to_bytes(self.tokens), b'\0', to_bytes(self.data))
        return hashlib.new(self.digest, b''.join(parts)).hexdigest()

    def _encode_ip(self, ip):
        return socket.inet_aton(ip)

    def _encode_ts(self, ts):
        return struct.pack('!I', ts)



import base64
from gzip import zlib
import hashlib
import hmac
import json
import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.ciphers import algorithms, Cipher, modes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

from auth_tkt.compat import to_bytes
from auth_tkt.ticket import AuthTkt


BACKEND = default_backend()


class DecryptionError(Exception):
    """Failed to decrypt. Can happen with wrong password, for example."""
    pass


class EncryptedAuthTkt(object):

    @staticmethod
    def from_data(
            authtkt_secret, payload_secret, uid, data=None, ip='0.0.0.0',
            tokens=(), base64=True, ts=None, encoding='utf-8'):
        data = json.dumps(data or {})
        data = _encrypt_userdata(
            to_bytes(data, encoding), to_bytes(payload_secret, encoding))
        auth_ticket = AuthTkt(
            authtkt_secret, uid, data, ip, tokens, base64, ts,
            encoding=encoding)
        return EncryptedAuthTkt(auth_ticket, payload_secret)

    def __init__(self, auth_ticket, payload_secret):
        self.auth_ticket = auth_ticket
        self._payload_secret = to_bytes(payload_secret)

    @property
    def uid(self):
        return self.auth_ticket.uid

    @property
    def data(self):
        return json.loads(_decrypt_userdata(
            self.auth_ticket.data, self._payload_secret).decode())

    def ticket(self):
        return self.auth_ticket.ticket()

    def cookie(self, name, **kwargs):
        return self.auth_ticket.cookie(name, **kwargs)

    def cookie_value(self):
        return self.auth_ticket.cookie_value()


def _derive_keys(secret, salt=None):
    if salt is None:
        salt = os.urandom(32)

    # derive 256 bit encryption key using the pbkdf2 standard
    key = PBKDF2HMAC(
        algorithm=hashes.SHA1(), length=32, salt=salt, iterations=1000,
        backend=BACKEND).derive(secret)

    # Derive encryption key and HMAC key from it
    # See Practical Cryptography section 8.4.1.
    hmacKey = hashlib.sha256(key + b'MAC').digest()
    encKey = hashlib.sha256(key + b'encrypt').digest()

    return hmacKey, encKey, salt


def _encrypt_userdata(cleartext, secret):
    hmacKey, encKey, salt = _derive_keys(secret)

    # get 128 bit random iv
    iv = os.urandom(16)

    # Add HMAC to cleartext so that we can check during decrypt if we got
    # the right cleartext back. We are doing sign-then-encrypt, which lets
    # us encrypt empty cleartext (otherwise we'd need to pad with some
    # string to encrypt). Practical Cryptography by Schneier & Ferguson
    # also recommends doing it in this order in section 8.2.
    mac = hmac.new(hmacKey, cleartext + iv + salt, hashlib.sha256).hexdigest()
    ciphertext = _encipher(
        zlib.compress(cleartext + to_bytes(mac)), encKey, iv, algorithms.AES)

    return (
        base64.b64encode(iv + salt + ciphertext)
        .strip()
        .replace(b'\n', b'')
    )


def _decrypt_userdata(ciphertext, secret):
    ciphertext = base64.b64decode(ciphertext)
    iv, salt, ciphertext = (
        ciphertext[:16], ciphertext[16:48], ciphertext[48:])

    hmacKey, encKey, salt = _derive_keys(secret, salt)

    # decrypt
    try:
        ret = zlib.decompress(_decipher(
            ciphertext, encKey, iv, algorithms.AES))
    except (zlib.error, ValueError) as e:
        raise DecryptionError(str(e))

    # Check MAC
    mac = ret[-64:]
    ret = ret[:-64]

    if hmac.new(hmacKey, ret + iv + salt,
                hashlib.sha256).hexdigest() != mac.decode():
        raise DecryptionError('HMAC does not match')
    return ret


def _decipher(ciphertext, key, iv, alg):
    return _cipherFilter(alg, key, iv, ciphertext, decrypt=True)


def _encipher(plaintext, key, iv, alg):
    return _cipherFilter(alg, key, iv, plaintext)


def _cipherFilter(alg, key, iv, input_, decrypt=False):
    key = key[:16]  # required for AES 128
    cipher = Cipher(alg(key), modes.OFB(iv), backend=BACKEND)
    cipher = cipher.decryptor() if decrypt else cipher.encryptor()
    return cipher.update(input_) + cipher.finalize()



import base64
import unittest

from auth_tkt.compat import base64decode
from auth_tkt.ticket import AuthTkt, validate


class AuthTktTests(unittest.TestCase):
    def test_construction_minimal(self):
        tkt = AuthTkt('secret', '123')
        self.assertEqual(tkt.secret, 'secret')
        self.assertEqual(tkt.uid, '123')
        self.assertEqual(tkt.data, '')
        self.assertEqual(tkt.ip, '0.0.0.0')
        self.assertEqual(tkt.tokens, '')
        self.assertTrue(tkt.base64)
        self.assertTrue(tkt.ts > 0)

    def test_construction_optional(self):
        tkt = AuthTkt('secret', '123', 'userdata', '127.0.0.1',
                      ('foo ', 'bar'), False, 9001)
        self.assertEqual(tkt.secret, 'secret')
        self.assertEqual(tkt.uid, '123')
        self.assertEqual(tkt.data, 'userdata')
        self.assertEqual(tkt.ip, '127.0.0.1')
        self.assertEqual(tkt.tokens, 'foo,bar')
        self.assertFalse(tkt.base64)
        self.assertEqual(tkt.ts, 9001)

    def construct(self, secret='secret', uid='123', **kwargs):
        if kwargs.get('tokens') is True:
            kwargs['tokens'] = ('foo', 'bar')
        kwargs.setdefault('data', 'userdata')
        kwargs.setdefault('ts', 9001)
        return AuthTkt(secret, uid, **kwargs)

    def test_cookie_value(self):
        tkt = self.construct()
        body = tkt.cookie_value()
        digest = '7f31d235ecc1a1c566ebd51469ed8a59'
        ip = '0000'  # 0.0.0.0
        ts = '2329'  # 9001
        id_ = '123'
        userdata = 'userdata'
        self.assertEqual(body, digest + ip + ts + id_ + '!' + userdata)

    def test_cookie_value_with_tokens(self):
        tkt = self.construct(tokens=True)
        body = tkt.cookie_value()
        digest = '575cd7937781c0636da95f0f4f423aef'
        ip = '0000'  # 0.0.0.0
        ts = '2329'  # 9001
        id_ = '123'
        tokens = 'foo,bar'
        userdata = 'userdata'
        self.assertEqual(body, digest + ip + ts + id_ + '!' + tokens + '!'
                         + userdata)

    def test_cookie(self):
        tkt = self.construct(base64=False)
        c = tkt.cookie('test_cookie', domain='example.com')['test_cookie']
        self.assertEqual(c.key, 'test_cookie')
        self.assertEqual(c['path'], '/')
        self.assertEqual(c['domain'], 'example.com')
        for key in ('expires', 'comment', 'max-age', 'secure', 'version',
                    'httponly'):
            self.assertFalse(c[key])
        self.assertEqual(c.value, tkt.cookie_value())

    def test_cookie_unicode(self):
        tkt = self.construct()
        cookies = tkt.cookie(u'test_cookie', domain=u'example.com')
        self.assertTrue('test_cookie' in cookies)

    def test_ticket(self):
        tkt = self.construct(base64=False)
        self.assertEqual(tkt.ticket(), tkt.cookie_value())

    def test_ticket_b64(self):
        tkt = self.construct()
        self.assertEqual(base64decode(tkt.ticket()), tkt.cookie_value())

    def test_construct_unicode(self):
        tkt = self.construct(u'secret', u'123', data=u'userdata',
                             ip=u'0.0.0.0', tokens=(u'foo', u'bar'),
                             base64=False)
        self.assertEqual(tkt.ticket(), tkt.cookie_value())


class AuthTktInternalTests(unittest.TestCase):
    def setUp(self):
        self.tkt = AuthTkt('secret', '123', 'userdata', '123.45.67.89',
                           ('foo', 'bar'), ts=9001)

    def test_digest0(self):
        self.assertEqual(self.tkt._digest0(),
                         '7c7ab37013af6e77759ec5e2f6129928')

    def test_encode_ip(self):
        self.assertEqual(self.tkt._encode_ip(self.tkt.ip), b'{-CY')

    def test_encode_ts(self):
        self.assertEqual(self.tkt._encode_ts(self.tkt.ts), b'\x00\x00#)')


class ValidateTests(unittest.TestCase):
    secret = 'secret'

    def test_garbage(self):
        self.assertFalse(validate('blergh', self.secret))

    def build_ticket(self, digest=b'575cd7937781c0636da95f0f4f423aef',
                     ip=b'0000', ts=b'2329', id_=b'123', tokens=b'foo,bar',
                     data=b'userdata', base64encode=False):
        if tokens:
            ticket = digest + ip + ts + id_ + b'!' + tokens + b'!' + data
        else:
            ticket = digest + ip + ts + id_ + b'!' + data
        if base64encode:
            ticket = b''.join(base64.b64encode(ticket).split())
        return ticket.decode('latin_1')

    def test_valid(self):
        body = self.build_ticket()
        tkt = validate(body, self.secret, timeout=0)
        self.assertTrue(tkt)
        self.assertTrue(isinstance(tkt, AuthTkt))

    def test_valid_b64(self):
        body = self.build_ticket(base64encode=True)
        self.assertTrue(validate(body, self.secret, timeout=0))

    def test_wrong_digest(self):
        body = self.build_ticket(digest=b'1234567890abcdef' * 2)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_wrong_digest_b64(self):
        body = self.build_ticket(
            digest=b'1234567890abcdef' * 2, base64encode=True)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_invalid_digest(self):
        body = self.build_ticket(digest=b'\x00\xff' * 16)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_invalid_digest_b64(self):
        body = self.build_ticket(digest=b'\x00\xff' * 16, base64encode=True)
        self.assertFalse(validate(
            body, self.secret, timeout=0, encoding='latin_1'))

    def test_wrong_ip(self):
        body = self.build_ticket(ip=b'1234')
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_wrong_ip_b64(self):
        body = self.build_ticket(ip=b'1234', base64encode=True)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_wrong_ts(self):
        body = self.build_ticket(ts=b'0000')
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_wrong_ts_b64(self):
        body = self.build_ticket(ts=b'0000', base64encode=True)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_timed_out(self):
        body = self.build_ticket()
        self.assertFalse(validate(body, self.secret))

    def test_wrong_id(self):
        body = self.build_ticket(id_=b'124')
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_wrong_id_b64(self):
        body = self.build_ticket(id_=b'124', base64encode=True)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_no_tokens(self):
        body = self.build_ticket(tokens=None)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_no_tokens_b64(self):
        body = self.build_ticket(tokens=None, base64encode=True)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_wrong_tokens(self):
        body = self.build_ticket(tokens=b'foo,baz')
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_wrong_tokens_b64(self):
        body = self.build_ticket(tokens=b'foo,baz', base64encode=True)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_wrong_userdata(self):
        body = self.build_ticket(data=b'!' * 32)
        self.assertFalse(validate(body, self.secret, timeout=0))

    def test_wrong_userdata_b64(self):
        body = self.build_ticket(data=b'!' * 32, base64encode=True)
        self.assertFalse(validate(body, self.secret, timeout=0))
        
    def test_various_digest_types(self):
        for digest in ('md5', 'sha256', 'sha512'):
            tkt = AuthTkt(self.secret, 'uid',
                digest=digest, base64=False)
            self.assertTrue(validate(tkt.cookie_value(), self.secret,
                digest=digest), "%s validation failed" % digest)



import re

from setuptools import setup

with open('auth_tkt/__init__.py') as init_py:
    metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", init_py.read()))

with open('README.md', 'r') as readme_file:
    long_description = readme_file.read()

setup(
    name='auth_tkt',
    version=metadata['version'],
    description=metadata['doc'],
    author='Yola',
    author_email='engineers@yola.com',
    license='MIT',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='http://github.com/yola/auth_tkt',
    packages=['auth_tkt'],
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    install_requires=['cryptography < 46']
)



from auth_tkt import compat
from unittest import TestCase


class Base64DecodeTestCase(TestCase):

    def test_returns_decoded_string(self):
        self.assertEqual(
            compat.base64decode('ZGVjb2RlZA=='), 'decoded')


class Base64EncodeTestCase(TestCase):

    def test_encodes_passed_string(self):
        self.assertEqual(
            compat.base64encode('decoded'), 'ZGVjb2RlZA==')


class ToBytesTestCase(TestCase):

    def test_returns_encoded_byte_string(self):
        returns = compat.to_bytes('test')
        self.assertIsInstance(returns, bytes)
        self.assertEqual(returns, b'test')

    def test_encodes_unicode_strings(self):
        self.assertEqual(compat.to_bytes(u'\u2603'), b'\xe2\x98\x83')
